package io.zipcoder.tc_spring_poll_application.controller;

import io.zipcoder.tc_spring_poll_application.domain.Option;
import io.zipcoder.tc_spring_poll_application.domain.Vote;
import io.zipcoder.tc_spring_poll_application.dtos.OptionCount;
import io.zipcoder.tc_spring_poll_application.dtos.VoteResult;
import io.zipcoder.tc_spring_poll_application.repositories.VoteRepository;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.Collection;
import java.util.Map;
import java.util.TreeMap;

@RestController
public class ComputeResultController {

    private final VoteRepository voteRepository;

    @Autowired
    public ComputeResultController(VoteRepository voteRepository) {
        this.voteRepository = voteRepository;
    }

    @GetMapping("/computeresult")
    public ResponseEntity<?> computeResult(@RequestParam Long pollId) {
        VoteResult voteResult = new VoteResult();
        Iterable<Vote> allVotes = voteRepository.findVotesByPoll(pollId);

        Map<Long, Integer> optionCounts = countOptions(allVotes);
        addOptionCountsToVoteResult(optionCounts, voteResult);

        return new ResponseEntity<>(voteResult, HttpStatus.OK);
    }

    private Map<Long, Integer> countOptions(Iterable<Vote> allVotes) {
        Map<Long, Integer> optionCounts = new TreeMap<>();

        for (Vote vote : allVotes) {
            if (!optionCounts.containsKey(vote.getOption().getId())) {
                optionCounts.put(vote.getOption().getId(), 1);
            } else {
                optionCounts.put(vote.getOption().getId(), optionCounts.get(vote.getOption().getId()) + 1);
            }
        }

        return optionCounts;
    }

    private void addOptionCountsToVoteResult(Map<Long, Integer> optionCounts, VoteResult voteResult) {
        for (Long id : optionCounts.keySet()) {
            OptionCount optionCount = new OptionCount();
            optionCount.setOptionId(id);
            optionCount.setCount(optionCounts.get(id));
            voteResult.getResults().add(optionCount);
        }
    }

}
